/******************************************************************************
* NOE_IRF2.G
* converts posterior mean of DSGE parameters into impulse response functions
*
* updated:  04/29/2005
******************************************************************************/
new;
closeall;
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
  __output = 1;
  _fcmptol = 1E-10;
  _verbose = 1;           /** 1 for verbose **/

#include pathspec.g;
#include noe_spec.g;

/** length of irf **/
nirf = 12;
ti   = seqa(1,1,nirf);

/********************************************************************
* load posterior mean of parameters
********************************************************************/
mhrun  = "1";
lpath  = parapath $+ "\\mhrun" $+ mhrun;

lpara  = lpath    $+ "\\o1p1d1t0pas";
open fhparas = ^lpara for read;
post_mean0 = readr(fhparas,1);
closeall fhparas;

lpara  = lpath    $+ "\\o1p8d1t0pas";
open fhparas = ^lpara for read;
post_mean1 = readr(fhparas,1);
closeall fhparas;


/********************************************************************
**      assign names to parameters
********************************************************************/

p_theta_H       = 1;
p_theta_F       = 2;
p_theta_Hstar   = 3;
p_theta_Fstar   = 4;
p_tau           = 5;
p_h             = 6;
p_alp           = 7;
p_eta           = 8;
p_psi1          = 9;
p_psi2          = 10;
p_psi3          = 11;
p_psi1star      = 12;
p_psi2star      = 13;
p_psi3star      = 14;
p_rhoA          = 15;
p_rhoR          = 16;
p_rhog          = 17;
p_rhoAstar      = 18;
p_rhoRstar      = 19;
p_rhogstar      = 20;
p_rhoz          = 21;
p_rr_steady     = 22;
p_gam_steady    = 23;
p_pi_steady     = 24;
p_pi_steadystar = 25;

/********************************************************************
* Perfect Pass-through version
********************************************************************/

post_mean2 = post_mean0;
post_mean2[p_theta_F]     = 0.00001;
post_mean2[p_theta_Hstar] = 0.00001;

post_mean0[p_psi3] = 0;
post_mean0[p_psi3star] = 0;

post_mean1[p_psi3] = 0;
post_mean1[p_psi3star] = 0;

post_mean2[p_psi3] = 0;
post_mean2[p_psi3star] = 0;


/********************************************************************
* Compute Impulse Response Functions
********************************************************************/

irf0 = dsgeirf(post_mean0,nirf,msel);
irf1 = dsgeirf(post_mean1,nirf,msel); 
irf2 = dsgeirf(post_mean2,nirf,msel); 

/********************************************************************
* Graph
********************************************************************/

/** name of observables **/
ystr     = "DelY " $|
           "Pi   " $|
           "R    " $|
           "DelY*" $|
           "Pi*  " $|
           "R*   " $|
           "DelE "
           ;

/** number of observables **/
ny = 7;

/** name of shocks **/
titlestr = "R   " $|
           "R*  " $|
           "A   " $|
           "A*  " $|
           "G   " $|
           "G*  " $|
           "z   " $|
           "DelE";

/** number of shocks **/
nep = 8;

/** shock multiplier **/
shmult = ones(nep,1) ;

graphset;
begwind;
margin(0,0,0.2,0.2);
window(nep,ny,0);
fonts("microb");
_ptitlht = 0.5;
_paxht = 0.4;
_pnumht = 0.35;
_pcolor = {1 4 6};
_protate = 0;
_plctrl = 0;
_plwidth = 5.5;
_psymsiz = 5;
_plegstr = "Imperfect PTh\000Perfect PTh";

sh_ind = 1;
do until sh_ind > nep;
/* iterate over shocks */

   y_ind = 1;
   do until y_ind > ny;
   /* iterate over endogenous variables */

      title(titlestr[sh_ind]);
      ylabel(ystr[y_ind]);

      xy(ti, shmult[sh_ind]*irf0[.,ny*(sh_ind-1)+y_ind]
           ~ shmult[sh_ind]*irf2[.,ny*(sh_ind-1)+y_ind]
           ~ shmult[sh_ind]*irf1[.,ny*(sh_ind-1)+y_ind]);

      nextwind;

      y_ind = y_ind + 1;
   endo;

   sh_ind = sh_ind + 1;
endo;

endwind;

/********************************************************************
* Procedure: DSGEIRF
********************************************************************/
proc (1) = dsgeirf(para,nirf,msel);
/* INPUT:
**   para   : parameter vector
**   nirf   : length of irf
** OUTPUT:
**   ssirf  : impulse response
**            the structural shocks.
**            0 if retcode=0
**            (nirf by 25)
*/
local sig_chol,impact,
      ny,nstate,nz,nep,cumsumidx,
      T1,TC,T0,TETA,RC,
      TT,QQ,RR,HH,DD,ZZ,VV,
      eq_pi,eq_r,eq_ygr,
      eq_pi_h,eq_r_h,eq_ygr_h,eq_pi_F,eq_r_F,eq_ygr_F,eq_e_del,
      v_pi,v_pistar,v_y,v_ystar,v_e_del,v_R,v_Rstar,
      v_z,v_ylag,v_ystarlag,
      e_R,e_Rstar,e_A,e_Astar,e_g,e_gstar,e_z,e_e_del,
      gam_steady,rr_steady,pi_steady,pi_steadystar,
      sigR,sigRstar,sigA,sigAstar,sigg,siggstar,sigz,sigedel,
      corr_RRstar,
      yyirfall,yyirf,ss,sh_ind,t;


/**********************************************************
**      Solve DSGE model
**********************************************************/
{T1,TC,T0,TETA,RC} = dsgesolv(para,msel);

/**********************************************************
**      System matrices for State-space
**********************************************************/
if msel[1]==0;       /** open economy **/

   /**********************************************************
   **      System matrices for State-space
   **********************************************************/

    /** observation indices **/
   eq_ygr_h = 1;
   eq_pi_h  = 2;
   eq_r_h   = 3;
   eq_ygr_F = 4;
   eq_pi_F  = 5;
   eq_r_F   = 6;
   eq_e_del = 7;

    /** number of observable variables **/
    ny = 7;

    /** index for cummulative response **/
    /** output response is cumulative **/
    cumsumidx = eq_ygr_h | eq_ygr_F;


   /** variable indices **/
   v_pi         = 1;
   v_pistar     = 4;
   v_y          = 13;
   v_ystar      = 14;
   v_e_del      = 18;
   v_R          = 21;
   v_Rstar      = 22;
   v_z          = 27;

   /** shock indices **/
   e_R     = 1;
   e_Rstar = 2;
   e_A     = 3;
   e_Astar = 4;
   e_g     = 5;
   e_gstar = 6;
   e_z     = 7;
   e_e_del = 8;

   /** names to parameters **/
   rr_steady     = para[22];
   gam_steady    = para[23];
   pi_steady     = para[24];
   pi_steadystar = para[25];
   sigA          = para[26];
   sigg          = para[27];
   sigR          = para[28];
   sigAstar      = para[29];
   siggstar      = para[30];
   sigRstar      = para[31];
   sigz          = para[32];
   sigedel       = para[33];
   corr_RRstar   = para[34];

   /*******************************************
   ** Transition equations with augmentation
   **
   ** z(t) = T1*z(t-1) + T0*e(t)
   ** x(t) = [z(t)',...]'
   **
   ** x(t) = TT*x(t-1) + RR*e(t)
   ** e(t) ~ iid N(0,QQ)
   *******************************************/
   nz  = cols(T1);
   nep = cols(T0);

   /** augmentation: v_ylag, v_ystarlag **/
   v_ylag = nz+1;
   v_ystarlag = nz+2;

   TT = zeros(nz+2,nz+2);
   TT[1:nz,1:nz]  = T1;
   TT[v_ylag,v_y] = 1;
   TT[v_ystarlag,v_ystar] = 1;

   QQ = zeros(nep,nep);
   QQ[e_R,e_R]         = sigR^2;
   QQ[e_Rstar,e_Rstar] = sigRstar^2;
   QQ[e_R,e_Rstar]     = sigR*sigRstar*corr_RRstar;
   QQ[e_A,e_A]         = sigA^2;
   QQ[e_Astar,e_Astar] = sigAstar^2;
   QQ[e_g,e_g]         = sigg^2;
   QQ[e_gstar,e_gstar] = siggstar^2;
   QQ[e_z,e_z]         = sigz^2;
   QQ[e_e_del,e_e_del] = sigedel^2;

   RR = T0|zeros(2,nep);

   /*******************************************
   ** Measurement equations
   **
   **  INF_US = pi_steady + 4*PI;
   **  R_US   = pi_steady + rr_steady + 4*R;
   **  Ygr_US = gam_steady + Y - Y(-1) + Z;
   **  INF_F  = pi_steadystar + 4*PI_star;
   **  R_F    = pi_steadystar + rr_steady + 4*R_star;
   **  Ygr_F  = gam_steady + Y_star - Y_star(-1) + Z;
   **  E_del  = e_del;
   **
   **  y(t) = DD + ZZ*x(t) + u(t)
   **  u(t) ~ N(0,HH)
   **
   **  cov(e(t),u(t)) = VV
   *******************************************/
   nstate = cols(TT);

   DD = zeros(ny,1);
   DD[eq_pi_h,1]  = pi_steady;
   DD[eq_r_h,1]   = pi_steady + rr_steady;
   DD[eq_ygr_h,1] = gam_steady;
   DD[eq_pi_f,1]  = pi_steadystar;
   DD[eq_r_f,1]   = pi_steadystar + rr_steady;
   DD[eq_ygr_f,1] = gam_steady;

   ZZ = zeros(ny,nstate);
   ZZ[eq_pi_h,v_pi]       =  4;

   ZZ[eq_r_h,v_R]         =  4;

   ZZ[eq_ygr_h,v_y]       =  1;
   ZZ[eq_ygr_h,v_ylag]    = -1;
   ZZ[eq_ygr_h,v_z]       =  1;

   ZZ[eq_pi_f,v_pistar]   =  4;

   ZZ[eq_r_f,v_Rstar]     =  4;

   ZZ[eq_ygr_f,v_ystar]   =  1;
   ZZ[eq_ygr_f,v_ystarlag]= -1;
   ZZ[eq_ygr_f,v_z]       =  1;

   ZZ[eq_e_del,v_e_del]   =  1;

   HH = 0;
   VV = zeros(nep,ny);

elseif msel[1]>0;    /** closed economy **/

   /** observation indices **/
   eq_ygr = 1;
   eq_pi  = 2;
   eq_r   = 3;

    /** number of observable variables **/
   ny = 3;

    /** index for cummulative response **/
    /** output response is cumulative **/
    cumsumidx = eq_ygr;

   /** variable indices **/
   v_pi      = 1;
   v_y       = 5;
   v_R       = 6;
   v_z       = 9;

   /** shock indices **/
   e_R  = 1;
   e_A  = 2;
   e_g  = 3;
   e_z  = 4;

   /** names to parameters **/
   rr_steady  = para[10];
   gam_steady = para[11];
   pi_steady  = para[12];
   sigA       = para[13];
   sigg       = para[14];
   sigR       = para[15];
   sigz       = para[16];

   /*******************************************
   ** Transition equations with augmentation
   **
   ** z(t) = T1*z(t-1) + T0*e(t)
   ** x(t) = [z(t)',...]'
   **
   ** x(t) = TT*x(t-1) + RR*e(t)
   ** e(t) ~ iid N(0,QQ)
   *******************************************/
   nz  = cols(T1);
   nep = cols(T0);

   /** augmentation: v_ylag **/
   v_ylag = nz+1;

   TT = zeros(nz+1,nz+1);
   TT[1:nz,1:nz]  = T1;
   TT[v_ylag,v_y] = 1;

   QQ = zeros(nep,nep);
   QQ[e_R,e_R] = sigR^2;
   QQ[e_A,e_A] = sigA^2;
   QQ[e_g,e_g] = sigg^2;
   QQ[e_z,e_z] = sigz^2;

   RR = T0|zeros(1,nep);

   /*******************************************
   ** Measurement equations
   **
   **  INF_US = pi_steady + 4*PI;
   **  R_US   = pi_steady + rr_steady + 4*R;
   **  Ygr_US = gam_steady + Y - Y(-1) + Z;
   **
   **  y(t) = DD + ZZ*x(t) + u(t)
   **  u(t) ~ N(0,HH)
   **
   **  cov(e(t),u(t)) = VV
   *******************************************/
   nstate = cols(TT);

   DD = zeros(ny,1);
   DD[eq_pi,1]  = pi_steady;
   DD[eq_r,1]   = pi_steady + rr_steady;
   DD[eq_ygr,1] = gam_steady;

   ZZ = zeros(ny,nstate);
   ZZ[eq_pi,v_pi]    =  4;
   ZZ[eq_r,v_R]      =  4;
   ZZ[eq_ygr,v_y]    =  1;
   ZZ[eq_ygr,v_ylag] = -1;
   ZZ[eq_ygr,v_z]    =  1;

   HH = 0;
   VV = zeros(nep,ny);

endif;

/**********************************************************
**   impulse response function
**********************************************************/

/** Choleski decomposition **/
/* sig_chol = chol(QQ); */
sig_chol = sqrt(QQ);


yyirfall = zeros(nirf,ny*nep);

sh_ind = 1;
do until sh_ind > nep;

   impact = sig_chol[.,sh_ind];
   yyirf  = zeros(nirf,ny);

   ss     = (RR*impact);
   yyirf[1,.] = (ZZ*ss)';

   t = 2;
   do until t > nirf;
      ss = TT*ss;
      yyirf[t,.] = (ZZ*ss)';
      t = t+1;
   endo;

   if (sh_ind == 1);
      yyirf = yyirf / yyirf[1,3];
   elseif (sh_ind == 2);
      yyirf = yyirf / yyirf[1,6];
   endif;   

   yyirf[.,cumsumidx] = cumsumc(yyirf[.,cumsumidx]);  
   yyirfall[.,1+ny*(sh_ind-1):ny*sh_ind] = yyirf;

   sh_ind = sh_ind+1;

endo;

retp(yyirfall);
endp;

#include noe_mod.src;
